<?php

declare(strict_types=1);

namespace Drupal\jsonapi_address\Resource;

use CommerceGuys\Addressing\Country\CountryRepositoryInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\jsonapi\JsonApiResource\ResourceObjectData;
use Drupal\jsonapi_address\CountryResourceBuilder;
use Drupal\jsonapi_resources\Resource\ResourceBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Routing\Route;

/**
 * Handles address country collection.
 */
class CountryCollection extends ResourceBase implements ContainerInjectionInterface {

  public function __construct(protected readonly CountryRepositoryInterface $countryRepository, protected readonly CountryResourceBuilder $resourceBuilder) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('address.country_repository'),
      $container->get('jsonapi_address.country_resource_builder')
    );
  }

  /**
   * Processes the country collection request.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   *
   * @return \Drupal\jsonapi\ResourceResponse
   *   The resource response.
   */
  public function process(Request $request) {
    $list = [];
    foreach ($this->countryRepository->getAll() as $country) {
      $list[] = $this->resourceBuilder->build($country);
    }

    $top_level_data = new ResourceObjectData($list, -1);
    return $this->createJsonapiResponse($top_level_data, $request);
  }

  /**
   * {@inheritdoc}
   */
  public function getRouteResourceTypes(Route $route, string $route_name): array {
    return [$this->resourceBuilder->getResourceType()];
  }

}
