<?php

declare(strict_types=1);

namespace Drupal\jsonapi_address\Resource;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\jsonapi_address\CountryResourceBuilder;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\jsonapi\JsonApiResource\ResourceObjectData;
use Drupal\jsonapi_resources\Resource\ResourceBase;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Routing\Route;
use CommerceGuys\Addressing\Country\Country;

/**
 * Handles address country resource.
 */
class CountrySingle extends ResourceBase implements ContainerInjectionInterface {

  public function __construct(protected readonly CountryResourceBuilder $resourceBuilder) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('jsonapi_address.country_resource_builder')
    );
  }

  /**
   * Processes a country resource request.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   * @param \CommerceGuys\Addressing\Country\Country $country
   *   The two-digit country code.
   *
   * @return \Drupal\jsonapi\ResourceResponse
   *   The resource response.
   */
  public function process(Request $request, Country $country) {
    $resource = $this->resourceBuilder->build($country);
    $top_level_data = new ResourceObjectData([$resource], 1);
    return $this->createJsonapiResponse($top_level_data, $request);
  }

  /**
   * {@inheritdoc}
   */
  public function getRouteResourceTypes(Route $route, string $route_name): array {
    return [$this->resourceBuilder->getResourceType()];
  }

}
