<?php

declare(strict_types=1);

namespace Drupal\jsonapi_address\Resource;

use CommerceGuys\Addressing\Country\Country;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\jsonapi\JsonApiResource\ResourceObjectData;
use Drupal\jsonapi_address\FormatResourceBuilder;
use Drupal\jsonapi_resources\Resource\ResourceBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Routing\Route;

/**
 *
 */
class Format extends ResourceBase implements ContainerInjectionInterface {

  public function __construct(protected readonly FormatResourceBuilder $resourceBuilder) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('jsonapi_address.format_resource_builder')
    );
  }

  /**
   * Processes an address format resource request for a given country.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   * @param \CommerceGuys\Addressing\Country\Country $country
   *   The country object.
   *
   * @return \Drupal\jsonapi\ResourceResponse
   *   The JSONAPI response.
   */
  public function process(Request $request, Country $country) {
    $resource = $this->resourceBuilder->build($country);
    $top_level_data = new ResourceObjectData([$resource], 1);
    return $this->createJsonapiResponse($top_level_data, $request);
  }

  /**
   * {@inheritdoc}
   */
  public function getRouteResourceTypes(Route $route, string $route_name): array {
    return [$this->resourceBuilder->getResourceType()];
  }

}
