<?php

declare(strict_types=1);

namespace Drupal\jsonapi_address\Resource;

use CommerceGuys\Addressing\Country\Country;
use CommerceGuys\Addressing\Country\CountryRepositoryInterface;
use CommerceGuys\Addressing\Subdivision\SubdivisionRepositoryInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\jsonapi\JsonApiResource\ResourceObjectData;
use Drupal\jsonapi_address\SubdivisionResourceBuilder;
use Drupal\jsonapi_resources\Resource\ResourceBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Routing\Route;

/**
 * Handles address a subdivision collection.
 */
class SubdivisionCollection extends ResourceBase implements ContainerInjectionInterface {

  public function __construct(protected readonly SubdivisionRepositoryInterface $subdivisionRepository, protected readonly SubdivisionResourceBuilder $resourceBuilder) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('address.subdivision_repository'),
      $container->get('jsonapi_address.subdivision_resource_builder')
    );

  }

  /**
   * Processes the subdivision collection request.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   * @param \CommerceGuys\Addressing\Country\Country $country
   *   The country object.
   *
   * @return \Drupal\jsonapi\ResourceResponse
   *   The resource response.
   */
  public function process(Request $request, Country $country) {
    $list = [];
    foreach ($this->subdivisionRepository->getAll([$country]) as $subdivision) {
      $list[] = $this->resourceBuilder->build($subdivision);
    }
    $top_level_data = new ResourceObjectData($list, -1);
    return $this->createJsonapiResponse($top_level_data, $request);
  }

  /**
   * {@inheritdoc}
   */
  public function getRouteResourceTypes(Route $route, string $route_name): array {
    return [$this->resourceBuilder->getResourceType()];
  }

}
