<?php

declare(strict_types=1);

namespace Drupal\jsonapi_address\Resource;

use CommerceGuys\Addressing\Subdivision\Subdivision;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\jsonapi\JsonApiResource\ResourceObjectData;
use Drupal\jsonapi_address\SubdivisionResourceBuilder;
use Drupal\jsonapi_resources\Resource\ResourceBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Routing\Route;

/**
 * Handles address subdivision resource.
 */
class SubdivisionSingle extends ResourceBase implements ContainerInjectionInterface {

  public function __construct(protected readonly SubdivisionResourceBuilder $resourceBuilder) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('jsonapi_address.subdivision_resource_builder'),
    );
  }

  /**
   * Generates a resource response for a subdivision.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   * @param \CommerceGuys\Addressing\Subdivision\Subdivision $subdivision
   *   The subdivision object.
   *
   * @return \Drupal\jsonapi\ResourceResponse
   *   Returns the resource response.
   */
  public function process(Request $request, Subdivision $subdivision) {
    $resource = $this->resourceBuilder->build($subdivision);
    $top_level_data = new ResourceObjectData([$resource], 1);
    return $this->createJsonapiResponse($top_level_data, $request);
  }

  /**
   * {@inheritdoc}
   */
  public function getRouteResourceTypes(Route $route, string $route_name): array {
    return [$this->resourceBuilder->getResourceType()];
  }

}
